package webui

import (
    "fmt"
	"log"
	"net/http"
    "os/exec"
    "runtime"
    "time"
    "github.com/go-co-op/gocron/v2"
    "gitea.armuli.eu/museum-digital/museum-digital-webdav-uploader/src/cli"
    "gitea.armuli.eu/museum-digital/museum-digital-webdav-uploader/src/configloader"
)

var config configloader.MDWebDavUploaderConfig
var setupRequired bool
var scheduler       gocron.Scheduler
var scheduledUpload gocron.Job

// Opens the application in the default browser.
func openInBrowser(port string) {

	var err error
    target := "localhost:" + port
	switch runtime.GOOS {
	case "linux":
		err = exec.Command("xdg-open", target).Start()
	case "windows":
		err = exec.Command("rundll32", "url.dll,FileProtocolHandler", target).Start()
	case "darwin":
		err = exec.Command("open", target).Start()
	default:
		err = fmt.Errorf("unsupported platform")
	}
	if err != nil {
		fmt.Println(err)
	}

}

func scheduleUploads() {

    s, err := gocron.NewScheduler()
    defer func() { _ = s.Shutdown() }()

    if err != nil {
        panic("Failed to set up scheduler")
    }

    j, jobErr := s.NewJob(
        gocron.DurationJob(
            3 * time.Hour,
        ),
        gocron.NewTask(
            func() {
                fmt.Println("Checking if an upload should be done")
                fmt.Println(time.Now().Format("20060102150405"))
                cli.Upload()
            },
        ),
    )
    if jobErr != nil {
        panic("Failed to set up scheduler job")
    }

    scheduledUpload = j
    s.Start()
    select {}

}

// Sets up the server and manages routing.
func Run() {

    config, setupRequired, _ = configloader.LoadFromFile("")

	// Bind callable URLs
	http.HandleFunc("/", serveAppShell)                         // Serve app shell
	http.HandleFunc("/get-settings", serveApiGetSettings)       // API for listing current settings
	http.HandleFunc("/get-translations", serveApiGetTranslations)       // API for listing current settings
	http.HandleFunc("/list-uploadable", serveApiListUploadable) // API to list uploadable files
	http.HandleFunc("/list-parsers", serveApiListParsers)       // API to list available parsers.
    http.HandleFunc("/update-settings", serveApiUpdateSettings) // API for updating settings
	http.HandleFunc("/trigger-upload", serveApiTriggerUpload)           // Serve page for specific files
	http.HandleFunc("/md-uploader.css", serveCss)        // Main CSS file.
	http.HandleFunc("/md-uploader.js", serveJs)          // Main JS file.
	http.HandleFunc("/logo.svg", serveLogo)              // Logo.

    port := "8080"
    go func() {

        // Set port to listen on
        fmt.Println("Server started at port " + string(port))
        err := http.ListenAndServe(":"+port, nil)
        if err != nil {
            log.Fatal("ListenAndServe: ", err)
        }

    }();
    go func() {
        time.Sleep(5)
        openInBrowser(port)
    }();
    go scheduleUploads()

    select{}

}
